;; Copyright 2007, 2008, 2009, 2010, 2012, 2013, 2014, 2015, 2016, 2017 Kevin Ryde

;; This file is part of xtide-el.
;;
;; xtide-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; xtide-el is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with xtide-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;;-----------------------------------------------------------------------------
;; xtide-run-insert-image

(load (expand-file-name "../xtide.el"))
(progn
  (switch-to-buffer "x")
  (erase-buffer)
  (xtide-run-insert-image))

(let ((extent (make-extent (point-min) (+ 10 (point-min)))))
  (set-extent-properties extent (list 'invisible t)))

(extent-at (point))
(xtide)


;; nil     ;; DIR
;; nil     ;; DEFAULT-FILENAME
;; nil     ;; no MUSTMATCH
;; 
;;     ".txt" ".png"))))

;; 
;; '(let ((orig-buffer (current-buffer)))
;;    (with-temp-buffer
;;      (let ((temp-buffer (current-buffer)))
;;        (xtide-set-buffer-multibyte nil) ;; unibyte temp buffer
;; 
;;        (with-current-buffer orig-buffer
;;          (let ((status (xtide-run "-fp" temp-buffer)))
;;            (if (eq 0 status)
;;                (xtide-insert-image
;;                 `(image :type png
;;                         :data ,(with-current-buffer temp-buffer
;;                                  (buffer-string))))
;;              (insert status))))))))

(defun xtide-insert-image (image-spec)
  "An internal part of xtide.el.
As per Emacs `insert-image', working in XEmacs too.
IMAGE-SPEC is a list like (image :type png :data \"...\")"
  (cond ((eval-when-compile (fboundp 'insert-image))
         ;; emacs
         (insert-image image-spec))

        ((eval-when-compile (fboundp 'make-glyph))
         ;; xemacs21
         (insert " ")
         (setq image-spec (copy-sequence (cdr image-spec)))
         (let* ((type   (plist-get image-spec :type))
                (glyph  (make-glyph (apply 'vector type
                                           (plist-remprop image-spec :type))))
                (extent (make-extent (1- (point)) (point))))
           (set-extent-property extent 'end-glyph glyph)))

        (t
         (error "No insert-image in this Emacs"))))





;;-----------------------------------------------------------------------------
;; xtide-great-circle-distance

(xtide-great-circle-distance 0 0 (/ pi 2) (/ pi 2))
(xtide-great-circle-distance 0 0 pi 0)
(xtide-great-circle-distance 0 0 (* 2 pi) 0)

(xtide-great-circle-distance -.1 -.3 .1 .3)

(xtide-location-line-lat-long "Some, where  Ref 33.9833° S, 151.2167° E")
(xtide-location-line-lat-long "Botany Bay, Australia        Ref 33.9833° N, 151.2167° E")
(xtide-location-line-lat-long "Botany Bay, Australia        Ref 33.9833° S, 151.2167° W")
(xtide-location-line-lat-long "Botany Bay, Australia        Ref 33.9833° N, 151.2167° W")


;;-----------------------------------------------------------------------------
;; menu entry

(calendar)


  (dolist (buffer (buffer-list))
    (with-current-buffer buffer
      (if (derived-mode-p 'calendar-mode)
          (add-menu-button '("Calendar") ["Tides" xtide-calendar-tides]))))


;;-----------------------------------------------------------------------------
;; percentage

      ;;     height before after)
      ;; (with-temp-buffer
      ;;   (call-process "tide" nil
      ;;                 (list t nil)  ;; output
      ;;                 nil           ;; redisplay
      ;;                 "-mr"  ;; "raw" output
      ;;                 "-z"   ;; GMT time values
      ;;                 "-b" (xtide-format-time-gmt time)
      ;;                 "-e" (xtide-format-time-gmt
      ;;                       (time-add time (seconds-to-time 61)))
      ;;                 "-l" (xtide-location-value)))
      ;; (goto-char (point-min))
      ;; (setq height (and (re-search-forward "^[0-9]+ \\([0-9.]+\\)")
      ;;                   (string-to-number (match-string 1))))

(progn
  ;; t=0 to pi/2 is time 0 to 6hours
  ;; h(t) = sine curve = sin(t) * 100
  ;; h'(t) = cos(t) * 100 maximum h'(t)=100 at pi/4*24hrs
  ;; change by 1% in 6hrs/100 = 3.6 minutes, much longer near peaks

  (require 'xtide)

  (defface xtide-rising
    `(;; plain "green" is too light to see against a white background
      (((class color) (background light))
       (:foreground "green4"))
      (((class color))
       (:foreground "green")))
    "Face for a rising tide."
    :group 'xtide)

  (defface xtide-falling
    `((((class color))
       (:foreground "red")))
    "Face for a falling tide."
    :group 'xtide)

  (defun xtide-format-time-gmt (time)
    (format-time-string "%Y-%m-%d %H:%M" time t)) ;; GMT

  (defun xtide-height-percent (&optional time location)
    (unless time (setq time (current-time)))
    (unless location (setq location (xtide-location-value)))

    (with-temp-buffer
      (call-process "tide" nil
                    (list t nil)  ;; output
                    nil           ;; redisplay
                    "-mr"  ;; "raw" output
                    "-z"   ;; time args are GMT
                    "-b" (xtide-format-time-gmt
                          (time-add time (seconds-to-time -86400)))
                    "-e" (xtide-format-time-gmt
                          (time-add time (seconds-to-time 86400)))
                    "-s" "00:02"  ;; steps
                    "-l" location)

      ;; lines like "1380093540 0.493632"
      (goto-char (point-min))
      (let ((time-seconds   (time-to-seconds time))
            (prev-height    nil)
            (prev-rising    nil)
            (extreme        nil)
            (before-extreme nil)
            (count          0))
        (while (and (not (and before-extreme extreme))
                    (re-search-forward "^\\(^[0-9]+\\) \\([0-9.]+\\)" nil t))
          (incf count)
          (let* ((this-seconds (string-to-number (match-string 1)))
                 (this-height  (string-to-number (match-string 2)))
                 (rising       (and prev-height (> this-height prev-height))))
            (with-current-buffer (get-buffer-create "xx")
              (insert (format "%S   %S %S   %s %s\n"
                              this-seconds this-height prev-height
                              rising prev-rising)))
            (unless (eq rising prev-rising)
              (with-current-buffer (get-buffer-create "xx") (insert "\n"))
              ;; switched from rising to falling or vice-versa, record extreme
              (setq extreme prev-height)
              (setq prev-rising rising))

            (when (and (not before-extreme)
                       (>= this-seconds time-seconds))
              (with-current-buffer (get-buffer-create "xx") (insert "\nafter\n"))
              (setq current-height this-height)
              (setq before-extreme extreme)
              (setq extreme nil))

            (setq prev-height this-height)))

        (and before-extreme
             current-height
             extreme
             (let* ((min   (min before-extreme extreme))
                    (max   (max before-extreme extreme))
                    (range (- max min)))
               (and (not (zerop range))
                    (propertize (number-to-string (round (/ (- current-height min) range 0.01)))
                                'face (if (> extreme before-extreme)
                                          'xtide-rising
                                        'xtide-falling)))))

        ;; (list before-extreme current-height extreme count
        ;;       )
        )))

  (defvar xtide-minor-mode-timer nil)

  (defvar xtide-minor-mode-string-cache nil)
  (defun xtide-minor-mode-string ()
    (let* ((location (xtide-location-value)))
      (or (cdr (assoc location xtide-minor-mode-string-cache))
          (let ((height (xtide-height-percent (current-time) location)))
            (push (cons location height) xtide-minor-mode-string-cache)
            height))))

;;;###autoload (put 'xtide-minor-mode-string 'risky-local-variable t)

  (defun xtide-minor-mode-timer-handler ()
    (setq xtide-minor-mode-string-cache nil)
    ;; redisplay  now if no input pending
    (sit-for 0))

  (define-minor-mode xtide-minor-mode
    "Toggle display of tide height in the mode line.
With a prefix argument ARG, enable Display Time mode if ARG is
positive, and disable it otherwise.  If called from Lisp, enable
it if ARG is omitted or nil.

When XTide minor mode is enabled it updates every minute."

    :global t
    :group  'display-time

    (if xtide-minor-mode-timer (cancel-timer xtide-minor-mode-timer))
    (setq xtide-minor-mode-timer nil)
    (setq xtide-minor-mode-string "")

    (when xtide-minor-mode-mode
      (unless global-mode-string (setq global-mode-string '("")))
      (add-to-list 'global-mode-string
                   '(:eval (xtide-minor-mode-string))
                   t) ;; append

      (setq xtide-minor-mode-timer
            (run-at-time nil ;; first trigger now
                         60  ;;
                         'xtide-minor-mode-timer-handler))
      (xtide-minor-mode-update)))

  (progn
    (switch-to-buffer "tt")
    (erase-buffer)
    (let* ((now  (current-time))
           (time (time-add now  (seconds-to-time (* -6 3600))))
           (end  (time-add time (seconds-to-time (* 12 3600)))))
      (while (time-less-p time end)
        (insert (format-time-string "%Y-%m-%d %H:%M" time))
        (insert (format "  %s\n" (xtide-height-percent time)))
        (redisplay)
        (setq time (time-add time (seconds-to-time 60)))))))


(insert (format-time-string "%Y-%m-%d %H:%M" (current-time) t))
2013-09-25 07:41
(insert (format "%S" (time-to-seconds (current-time))))
1380094868.4778087
(call-process "perl" nil t nil "-e" "print time()")
1380095040

(progn
  (defun xtide-height-now ()
    (with-temp-buffer
      (let* ((start-time (current-time))
             (end-time   (time-add start-time (seconds-to-time 61))))
        (call-process "tide" nil
                      (list t nil)  ;; output
                      nil           ;; redisplay
                      "-mr"  ;; "raw" output
                      "-z"   ;; GMT time values
                      "-b" (format-time-string "%Y-%m-%d %H:%M" start-time
                                               t) ;; GMT
                      "-e" (format-time-string "%Y-%m-%d %H:%M" end-time
                                               t) ;; GMT
                      "-l" (xtide-location-value)))
      (goto-char (point-min))
      (and (re-search-forward "^[0-9]+ \\([0-9.]+\\)")
           (string-to-number (match-string 1))))))



;;-----------------------------------------------------------------------------
;; desktop.el

(xtide)
(xtide-location-select)
(desktop-save "/tmp" t)
(desktop-read "/tmp")
(find-file "/tmp/.emacs.desktop")


(with-current-buffer "*xtide*"
  (set (make-local-variable 'desktop-save-buffer)
       'xtide-desktop-buffer-misc-data))
(with-current-buffer "*xtide*"
  (set (make-local-variable 'desktop-save-buffer)
       'ignore)
  (add-to-list 'desktop-locals-to-save 'xtide-location)
  (add-to-list 'desktop-locals-to-save 'xtide-output-mode))

(desktop-revert)
(desktop-clear)

;;-----------------------------------------------------------------------------
;; /usr/share/emacs/site-lisp/windows/revive.el
;; ~/.revive.el

(let ((print-level nil)
      (print-length nil)
      (eval-expression-print-level nil)
      (eval-expression-print-length nil))
  (require 'revive)
  (split-window)
;;  (info "(emacs)")
  (xtide)
  (save-current-configuration)
  (sit-for 1)
  (wipe)
  (resume))

(eval-after-load "revive"
  '(progn
     (add-to-list 'revive:major-mode-command-alist
                  '(xtide-mode . xtide-revive))
     (add-to-list 'revive:save-variables-mode-local-default
                  '(xtide-mode xtide-location
                               ;; ENHANCE-ME: (xtide) overwrites
                               ;; xtide-output-mode so a restore of that
                               ;; doesn't work
                               xtide-output-mode))
     ;; (add-to-list 'revive:major-mode-command-alist
     ;;              '(xtide-location-mode . xtide-location-select))
     ))

(my-remove-from-list 'revive:major-mode-command-alist
                     '(xtide-mode . xtide-revive))

;;;###autoload
(defun xtide-revive ()
  "Restore `xtide-mode' buffer for revive.el.
This function is designed to be called by revive.el from an entry
in `revive:major-mode-command-alist'.

The `xtide-location' and `xtide-output-mode' in the xtide buffer
are restored.  The time displayed is now (not the time when
`save-current-configuration' was done)."

  (let ((buffer (get-buffer-create (revive:prop-buffer-name x))))
    (set-buffer buffer)
    (xtide-mode)
    (setq xtide-location    (cdr (assq 'xtide-location
                                       (revive:prop-varlist x))))
    (setq xtide-output-mode (cdr (assq 'xtide-output-mode
                                       (revive:prop-varlist x))))
    (xtide-mode-redraw)))

(setq debug-on-error nil)
(pp (revive:buffer-property-list) (current-buffer))

(progn
  (save-current-configuration)
  (sit-for 1)
  (wipe)
  (resume))


;;-----------------------------------------------------------------------------
(progn
  (add-to-list 'load-path (expand-file-name ".."))
  (setq xtide-location nil)
  (autoload 'xtide "xtide" nil t)
  (xtide))
(xtide-location-mode)

(locate-library "derived")
(find-file "/so/emacs20/emacs-20.7/inst/share/emacs/20.7/lisp/derived.el")


(progn
  (require 'cl)
  (require 'poe))
make-temp-file

(setq debug-on-error t)
(progn
  (setq calendar-latitude 0)
  (setq calendar-longitude 0))

(add-hook 'xtide-mode-hook 'turn-on-font-lock)

(customize-group 'xtide)

(progn
  (setq xtide-location nil)
  (setenv "XTIDE_DEFAULT_LOCATION" nil)
  (xtide))
(progn
  (setq xtide-location nil)
  (setenv "XTIDE_DEFAULT_LOCATION" "Aransas Channel, Texas")
  (xtide))
(progn
  (setq xtide-location nil)
  (setenv "XTIDE_DEFAULT_LOCATION" "Bogosity ")
  (xtide))

(setq xtide-run-switches '("-fc" "SeaGreen"))

;;-----------------
(eval-after-load "calendar"
  '(dolist (keyseq '([menu-bar moon]       ;; emacs21,22
                     [menu-bar Sun/Moon])) ;; emacs23
     (if (lookup-key calendar-mode-map keyseq)
         (easy-menu-add-item
          calendar-mode-map
          (mapcar 'symbol-name keyseq)
          `["Tides" xtide-calendar-tides
            :help "View tides for the selected day using XTide."]))))

(progn
  (require 'calendar)
  (add-to-list 'load-path (expand-file-name ".."))
  (require 'xtide)
  (calendar))
(lookup-key calendar-mode-map [menu-bar Sun/Moon])
(easy-menu-add-item calendar-mode-map
		    (if (lookup-key calendar-mode-map [menu-bar moon])
			'("menu-bar" "moon") ;; emacs21,22
		      '("menu-bar" "Sun/Moon")) ;; emacs23
		    `["Tides" xtide-calendar-tides
		      ,@(xtide-easy-menu-help "View tides for the selected day using XTide.")])

(setq debug-on-error t)
(debug-on-entry 'add-menu-button)
(cancel-debug-on-entry 'add-menu-button)
(easy-menu-add-item nil ;; calendar-mode-map
		    '("menu-bar" "Calendar")
		    ["XXTides" xtide-calendar-tides])
(easy-menu-add-item calendar-mode-map
		    '("Calendar")
		    ["XXTides" xtide-calendar-tides])

(easy-menu-add-item cal-menu-sunmoon-menu
                       nil
                       `["Tides" xtide-calendar-tides
                         ,@(xtide-easy-menu-help "View tides for the selected day using XTide.")])

    '(let ((keymap (or (lookup-key 
                                   [menu-bar moon]) ;; emacs21,22
                       (lookup-key calendar-mode-map
                                   [menu-bar Sun/Moon])))) ;; emacs23
       (define-key-after keymap [xtide]
   
         t)))) ;; add at end of keymap

;;-----------------
;; compile

(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-byte-compile))
(progn
  (add-to-list 'load-path (expand-file-name "."))
  (require 'my-byte-compile)
  (my-show-autoloads))

(progn
  (elint-initialize)
  (find-file "../xtide.el")
  (elint-current-buffer))

;;-----------------------------------------------------------------------------

(let ((elfile (expand-file-name "../xtide.el"))
      (tempfile "/tmp/x.el")
      (print-level nil))
  (require 'autoload)
  (find-file tempfile)
  (erase-buffer)
  (generate-file-autoloads elfile)
  (save-buffer)
  (byte-compile-file tempfile)
  (delete-file (concat tempfile "c"))
  (delete-file tempfile))

;;-----------------------------------------------------------------------------
;; set-time-zone-rule

(while t
  (set-time-zone-rule "ABC+1")
  (set-time-zone-rule "DEF+2"))

(while t
  (set-time-zone-rule "ABC+1")
  (setenv "TZ" nil))

(progn
  (set-time-zone-rule "GMT+0")
  (getenv "TZ"))


;;-----------------------------------------------------------------------------

(xtide-location-time-zone "Geelong")
(xtide-location-time-zone "Severn")
(xtide-location-time-zone "Xyz")
(setq xtide-location-time-zone-cache nil)

(xtide-time-string (current-time))
(setq xtide-location "Geelong")
(setq xtide-location "Severn")
(setq xtide-location "Boston")


(make-autoload
 '(defgroup xtide nil "Xtide."
    :prefix "xtide-"
    :group 'applications
    :link  '(url-link :tag "xtide.el home page"
                      "http://user42.tuxfamily.org/xtide/index.html")
    :link  '(url-link :tag "XTide home page"
                      "http://www.flaterco.com/xtide"))
 "xtide.el")


(let ((groupname "mine")
      (file  "my-file.el"))
  `(let ((loads (get ',groupname 'custom-loads)))
     (if (member ',file loads) nil
       (put ',groupname 'custom-loads (cons ',file loads))))
  ;; `(xlet (get ',groupname 'custom-loads)))
  ))


`(let ((x (symbol-name 'foo))) x)

`(identity 'foo)
`(identity (identity 'foo))
`(identity (identity (identity 'foo)))

`(let ((x (identity 'foo))) x)

`(let ((x (identity "foo"))) x)
`(let ((x (identity "foo" 'bar))) x)
`(let ((x (identity 'foo 'bar))) x)

(defmacro mymacro (arg)
  `(let ((x (symbol-name ,arg)))
     (message "%s" x)))
(mymacro 'hi)
(mymacro "hi")


;;-----------------------------------------------------------------------------
;; unibyte buffer

(progn
  (with-current-buffer (get-buffer-create "xx")
    (let ((m   (multibyte-string-p (buffer-string)))
          u)
      (set-buffer-multibyte nil) ;; unibyte temp buffer
      (setq u (multibyte-string-p (buffer-string)))
      (kill-buffer (current-buffer))
      (message "%S %S" m u))))

;;-----------------------------------------------------------------------------
;; customize

(defcustom foo "global"
  "String."
  :type  'string)

(customize-variable 'foo)
(set (make-local-variable 'foo) "local")


;;-----------------------------------------------------------------------------
;; xtide-location-value

(progn
  (byte-compile 'xtide-location-value)
  (disassemble 'xtide-location-value))

  (car (delq nil
             (mapcar (lambda (str)
                       (and str
                            (progn
                              ;; leading white
                              (if (string-match "\\`\\(\\s-\\|\n\\)+" str)
                                  (setq str (substring str (match-end 0))))
                              ;; trailing white
                              (if (string-match "\\(\\s-\\|\n\\)+\\'" str)
                                  (setq str (substring str 0 (match-beginning 0))))
                              (and (not (string-equal "" str))
                                   str))))

                     (list xtide-location
                           (getenv "XTIDE_DEFAULT_LOCATION"))))))

;;-----------------------------------------------------------------------------
;; xtide-location-mode bookmark

(progn
  (xtide-location-select)
  bookmark-make-record-function)

(progn
  (xtide-location-select)
  (goto-char (point-min))
  (forward-line 100)
  (forward-char 20)
  (bookmark-set "d-xtide")
  (forward-line -100)
  (kill-buffer (current-buffer))
  (bookmark-jump "d-xtide")
  (bookmark-get-bookmark "d-xtide")
  )

;; what a bookmark looks like
(pp (save-window-excursion
      (xtide-location-select)
      (forward-line 100)
      (forward-char 20)
      (xtide-location-bookmark-make-record))
    (current-buffer))
("Kumeon Bay, British Columbia (2)"
 (location . "Kumeon Bay, British Columbia (2)")
 (column . 20)
 (handler . xtide-location-bookmark-jump)
 (front-context-string . "Columbia (2)    ")
 (rear-context-string . "on Bay, British ")
 (position . 554182))

(pp (bookmark-make-record-default 'point-only)
    (current-buffer))

(setq my-bmk '("Kumeon Bay, British Columbia (2)"
               (location . "Kumeon Bay, British Columbia (2)")
               (column . 20)
               (handler . xtide-location-bookmark-jump)
               (front-context-string . "Columbia (2)    ")
               (rear-context-string . "on Bay, British ")
               (position . 554182)))
(bookmark-get-bookmark-record my-bmk)
(bookmark-default-handler
 (nconc (list ""
              (cons 'buffer (current-buffer)))
        (bookmark-get-bookmark-record my-bmk)))


;;-----------------------------------------------------------------------------
;; xtide-mode bookmark.el tie-in

(progn
  (xtide)
  (bookmark-set "d-xtide")
  (kill-buffer (current-buffer))
  (bookmark-jump "d-xtide")
  (bookmark-get-bookmark "d-xtide")
  )

;; (cons 'time         xtide-time)
;; xtide-time         (bookmark-prop-get bmk 'time)



;;-----------------------------------------------------------------------------
defgroup
(get 'xtide 'custom-links)

;;-----------------------------------------------------------------------------
;; macro expansion

(defmacro foo (arg)
  `(quote ,arg))
(foo 123)
(foo (bar))
(defmacro bar ()
  123)
(bar)


;;-----------------------------------------------------------------------------
;; `pure' is not safe

(put 'xtide-great-circle-distance 'pure t)
(put 'xtide-great-circle-distance 'safe-function t)
(unsafep '(xtide-great-circle-distance))

