;; Copyright 2009, 2019 Kevin Ryde

;; This file is part of xtide-el.
;;
;; xtide-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; xtide-el is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with xtide-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(defun my-fake-xtide-el-installed-handler (operation &rest args)
  "Pretend xtide.el is installed."
  (message "my-fake-xtide-el-installed-handler %S %S" operation args)
  (cond ((eq operation 'file-exists-p)
         t)
        ((eq operation 'expand-file-name)
         ;; absolute filename already, nothing to expand
         (car args))
        (t (error "oops, operation `%S' for xtide installed fakery"
                  operation))))

(defun my-run-test ()
  (let ((count 0))

    (if (not (boundp 'debian-emacs-flavor))
        (message "Skip, not a debian emacs")

      ;;-----------------------------------------------------------------------
      ;; emacsen-startup

      ;; dummy for debian/emacsen-startup under -q -no-site-file
      (unless (fboundp 'debian-pkg-add-load-path-item)
        (defalias 'debian-pkg-add-load-path-item 'identity))

      (let ((file-name-handler-alist
             (append `(("\\`/usr/share/emacs/site-lisp/xtide-el/"
                        . my-fake-xtide-el-installed-handler)
                       (,(concat "\\`/usr/share/"
                                 (symbol-name debian-emacs-flavor)
                                 "/site-lisp/xtide-el/")
                        . my-fake-xtide-el-installed-handler))
                     file-name-handler-alist)))
        (load (expand-file-name "../debian/emacsen-startup")))

      ;; make sure ahead of debian-pkg-add-load-path-item
      (add-to-list 'load-path (expand-file-name ".."))

      (require 'calendar)
      (unless (lookup-key calendar-mode-map "T")
        (error "no calendar T key defined"))
      (setq count (1+ count))

      (when (fboundp 'define-key-after) ;; not in xemacs21
        (unless (or (lookup-key calendar-mode-map
                                [menu-bar moon xtide]) ;; emacs21,22
                    (lookup-key calendar-mode-map
                                [menu-bar Sun/Moon xtide])) ;; emacs23
          (error "no calendar menu entry for xtide")))
      (setq count (1+ count))

      ;;-----------------------------------------------------------------------
      (message "  %d tests" count)
      count)))
